#
# Copyright (c) 2017 by QA Cafe.
# All Rights Reserved.
#

"""Module for accessing CDRouter Results."""

from marshmallow import Schema, fields, post_load
from .cdr_datetime import DateTime

class Options(object):
    def __init__(self, **kwargs):
        self.tags = kwargs.get('tags', None)
        self.skip_tests = kwargs.get('skip_tests', None)
        self.begin_at = kwargs.get('begin_at', None)
        self.end_at = kwargs.get('end_at', None)
        self.extra_cli_args = kwargs.get('extra_cli_args', None)

class OptionsSchema(Schema):
    tags = fields.List(fields.Str(), missing=None)
    skip_tests = fields.List(fields.Str(), missing=None)
    begin_at = fields.Str()
    end_at = fields.Str()
    extra_cli_args = fields.Str()

    @post_load
    def post_load(self, data):
        return Options(**data)

class Result(object):
    def __init__(self, **kwargs):
        self.id = kwargs.get('id', None)
        self.created = kwargs.get('created', None)
        self.updated = kwargs.get('updated', None)
        self.result = kwargs.get('result', None)
        self.status = kwargs.get('status', None)
        self.loops = kwargs.get('loops', None)
        self.tests = kwargs.get('tests', None)
        self.passed = kwargs.get('pass', None)
        self.fail = kwargs.get('fail', None)
        self.duration = kwargs.get('duration', None)
        self.size_on_disk = kwargs.get('size_on_disk', None)
        self.starred = kwargs.get('starred', None)
        self.archived = kwargs.get('archived', None)
        self.result_dir = kwargs.get('result_dir', None)
        self.agent_name = kwargs.get('agent_name', None)
        self.package_name = kwargs.get('package_name', None)
        self.device_name = kwargs.get('device_name', None)
        self.config_name = kwargs.get('config_name', None)
        self.package_id = kwargs.get('package_id', None)
        self.device_id = kwargs.get('device_id', None)
        self.config_id = kwargs.get('config_id', None)
        self.user_id = kwargs.get('user_id', None)
        self.note = kwargs.get('note', None)
        self.pause_message = kwargs.get('pause_message', None)
        self.build_info = kwargs.get('build_info', None)
        self.tags = kwargs.get('tags', None)
        self.testcases = kwargs.get('testcases', None)
        self.options = kwargs.get('options', None)

class ResultSchema(Schema):
    id = fields.Str()
    created = DateTime()
    updated = DateTime()
    result = fields.Str()
    status = fields.Str()
    loops = fields.Int()
    tests = fields.Int()
    passed = fields.Int(attribute='pass', load_from='pass', dump_to='pass')
    fail = fields.Int()
    duration = fields.Int()
    size_on_disk = fields.Int()
    starred = fields.Bool()
    archived = fields.Bool()
    result_dir = fields.Str()
    agent_name = fields.Str()
    package_name = fields.Str()
    device_name = fields.Str()
    config_name = fields.Str()
    package_id = fields.Str()
    device_id = fields.Str()
    config_id = fields.Str()
    user_id = fields.Str()
    note = fields.Str()
    pause_message = fields.Str(missing=None)
    build_info = fields.Str()
    tags = fields.List(fields.Str())
    testcases = fields.List(fields.Str())
    options = fields.Nested(OptionsSchema)

    @post_load
    def post_load(self, data):
        return Result(**data)

class ResultsService(object):
    """Service for accessing CDRouter Results."""

    RESOURCE = 'results'
    BASE = RESOURCE + '/'

    def __init__(self, service):
        self.service = service
        self.base = self.BASE

    def list(self, filter=None, sort=None, limit=None, page=None): # pylint: disable=redefined-builtin
        """Get a list of results."""
        schema = ResultSchema(exclude=('result', 'loops', 'tests', 'result_dir', 'agent_name', 'config_name', 'note', 'pause_message', 'testcases', 'options', 'build_info'))
        resp = self.service.list(self.base, filter, sort, limit, page)
        return self.service.decode(schema, resp, many=True)

    def list_csv(self, filter=None, sort=None, limit=None, page=None): # pylint: disable=redefined-builtin
        """Get a list of results as CSV."""
        return self.service.list(self.base, filter, sort, limit, page, format='csv')

    def get(self, id): # pylint: disable=invalid-name,redefined-builtin
        """Get a result."""
        schema = ResultSchema()
        resp = self.service.get_id(self.base, id)
        return self.service.decode(schema, resp)

    def stop(self, id, when=None): # pylint: disable=invalid-name,redefined-builtin
        """Stop a running result."""
        return self.service.post(self.base+str(id)+'/stop/', params={'when': when})

    def stop_end_of_test(self, id): # pylint: disable=invalid-name,redefined-builtin
        """Stop a running result at the end of the current test."""
        return self.stop(id, 'end-of-test')

    def stop_end_of_loop(self, id): # pylint: disable=invalid-name,redefined-builtin
        """Stop a running result at the end of the current loop."""
        return self.stop(id, 'end-of-loop')

    def pause(self, id, when=None): # pylint: disable=invalid-name,redefined-builtin
        """Pause a running result."""
        return self.service.post(self.base+str(id)+'/pause/', params={'when': when})

    def pause_end_of_test(self, id): # pylint: disable=invalid-name,redefined-builtin
        """Pause a running result at the end of the current test."""
        return self.pause(id, 'end-of-test')

    def pause_end_of_loop(self, id): # pylint: disable=invalid-name,redefined-builtin
        """Pause a running result at the end of the current loop."""
        return self.pause(id, 'end-of-loop')

    def unpause(self, id): # pylint: disable=invalid-name,redefined-builtin
        """Unpause a running result."""
        return self.service.post(self.base+str(id)+'/unpause/')

    def edit(self, resource):
        """Edit a result."""
        schema = ResultSchema()
        resp = self.service.edit(self.base, resource['id'], resource)
        return self.service.decode(schema, resp)

    def delete(self, id): # pylint: disable=invalid-name,redefined-builtin
        """Delete a result."""
        return self.service.delete_id(self.base, id)

    def get_shares(self, id): # pylint: disable=invalid-name,redefined-builtin
        """Get shares for a result."""
        return self.service.get_shares(self.base, id)

    def edit_shares(self, id, user_ids): # pylint: disable=invalid-name,redefined-builtin
        """Edit shares for a result."""
        return self.service.edit_shares(self.base, id, user_ids)

    def export(self, id, exclude_captures=False): # pylint: disable=invalid-name,redefined-builtin
        """Export a result."""
        return self.service.export(self.base, id, params={'exclude_captures': exclude_captures})

    def bulk_export(self, ids, exclude_captures=False):
        """Bulk export a set of results."""
        return self.service.bulk_export(self.base, ids, params={'exclude_captures': exclude_captures})

    def bulk_copy(self, ids):
        """Bulk copy a set of results."""
        schema = ResultSchema()
        return self.service.bulk_copy(self.base, self.RESOURCE, ids, schema)

    def bulk_edit(self, _fields, ids=None, filter=None, all=False): # pylint: disable=redefined-builtin
        """Bulk edit a set of results."""
        return self.service.bulk_edit(self.base, self.RESOURCE, _fields, ids=ids, filter=filter, all=all)

    def bulk_delete(self, ids=None, filter=None, all=False): # pylint: disable=redefined-builtin
        """Bulk delete a set of results."""
        return self.service.bulk_delete(self.base, self.RESOURCE, ids=ids, filter=filter, all=all)

    def all_stats(self):
        """Compute stats for all results."""
        return self.service.post(self.base, params={'stats': 'all'})

    def set_stats(self, ids):
        """Compute stats for a set of results."""
        return self.service.post(self.base, params={'stats': 'set'}, json=[{'id': str(x)} for x in ids])

    def single_stats(self, id): # pylint: disable=invalid-name,redefined-builtin
        """Compute stats for a result."""
        return self.service.get(self.base+str(id)+'/', params={'stats': 'all'})

    def list_logdir(self, id, filter=None, sort=None): # pylint: disable=invalid-name,redefined-builtin
        """Get a list of logdir files."""
        return self.service.list(self.base+str(id)+'/logdir/', filter, sort)

    def get_logdir_file(self, id, filename): # pylint: disable=invalid-name,redefined-builtin
        """Download a logdir file."""
        return self.service.get(self.base+str(id)+'/logdir/'+filename+'/')

    def download_logdir_archive(self, id, filename, format='zip', exclude_captures=False): # pylint: disable=invalid-name,redefined-builtin
        """Download logdir archive in tgz or zip format."""
        return self.service.get(self.base+str(id)+'/logdir/'+filename+'/', params={'format': format, 'exclude_captures': exclude_captures})

    def get_test_metric(self, id, name, metric, format=None): # pylint: disable=invalid-name,redefined-builtin
        """Get a test metric."""
        return self.service.get(self.base+str(id)+'/metrics/'+name+'/'+metric+'/',
                                params={'format': format})

    def get_test_metric_csv(self, id, name, metric): # pylint: disable=invalid-name,redefined-builtin
        """Get a test metric as CSV."""
        return self.get_test_metric(id, name, metric, format='csv')



#
# Copyright (c) 2017 by QA Cafe.
# All Rights Reserved.
#

"""Module for accessing CDRouter Configs."""

from marshmallow import Schema, fields, post_load
from .cdr_datetime import DateTime

class ConfigError(object):
    def __init__(self, **kwargs):
        self.lines = kwargs.get('lines', None)
        self.error = kwargs.get('error', None)

class ConfigErrorSchema(Schema):
    lines = fields.List(fields.Str())
    error = fields.Str()

    @post_load
    def post_load(self, data):
        return ConfigError(**data)

class CheckConfig(object):
    def __init__(self, **kwargs):
        self.errors = kwargs.get('errors', None)

class CheckConfigSchema(Schema):
    errors = fields.Nested(ConfigErrorSchema, many=True)

    @post_load
    def post_load(self, data):
        return CheckConfig(**data)

class UpgradeConfig(object):
    def __init__(self, **kwargs):
        self.success = kwargs.get('success', None)
        self.output = kwargs.get('output', None)

class UpgradeConfigSchema(Schema):
    success = fields.Bool()
    output = fields.Str()

    @post_load
    def post_load(self, data):
        return UpgradeConfig(**data)

class Networks(object):
    def __init__(self, **kwargs):
        self.name = kwargs.get('name', None)
        self.type = kwargs.get('type', None)
        self.side = kwargs.get('side', None)
        self.title = kwargs.get('title', None)
        self.children = kwargs.get('children', None)

class NetworksSchema(Schema):
    name = fields.Str()
    type = fields.Str()
    side = fields.Str()
    title = fields.Str()
    children = fields.Nested('self', many=True)

    @post_load
    def post_load(self, data):
        return Networks(**data)

class Testvar(object):
    def __init__(self, **kwargs):
        self.group = kwargs.get('group', None)
        self.name = kwargs.get('name', None)
        self.value = kwargs.get('value', None)
        self.default = kwargs.get('default', None)
        self.isdefault = kwargs.get('isdefault', None)
        self.line = kwargs.get('line', None)

class TestvarSchema(Schema):
    group = fields.Str()
    name = fields.Str()
    value = fields.Str()
    default = fields.Str()
    isdefault = fields.Bool()
    line = fields.Int()

    @post_load
    def post_load(self, data):
        return Testvar(**data)

class Config(object):
    def __init__(self, **kwargs):
        self.id = kwargs.get('id', None)
        self.name = kwargs.get('name', None)
        self.description = kwargs.get('description', None)
        self.created = kwargs.get('created', None)
        self.updated = kwargs.get('updated', None)
        self.contents = kwargs.get('contents', None)
        self.user_id = kwargs.get('user_id', None)
        self.result_id = kwargs.get('result_id', None)
        self.tags = kwargs.get('tags', None)
        self.note = kwargs.get('note', None)

class ConfigSchema(Schema):
    id = fields.Str()
    name = fields.Str()
    description = fields.Str()
    created = DateTime()
    updated = DateTime()
    contents = fields.Str()
    user_id = fields.Str()
    result_id = fields.Str(missing=None)
    tags = fields.List(fields.Str())
    note = fields.Str()

    @post_load
    def post_load(self, data):
        return Config(**data)

class ConfigsService(object):
    """Service for accessing CDRouter Configs."""

    RESOURCE = 'configs'
    BASE = RESOURCE + '/'

    def __init__(self, service):
        self.service = service
        self.base = self.BASE

    def list(self, filter=None, sort=None, limit=None, page=None): # pylint: disable=redefined-builtin
        """Get a list of configs."""
        schema = ConfigSchema(exclude=('contents', 'note'))
        resp = self.service.list(self.base, filter, sort, limit, page)
        return self.service.decode(schema, resp, many=True)

    def get_new(self):
        """Get output of cdrouter-cli -new-config."""
        return self.service.get(self.base, params={'template': 'default'})

    def get(self, id, format=None): # pylint: disable=invalid-name,redefined-builtin
        """Get a config."""
        schema = ConfigSchema()
        resp = self.service.get_id(self.base, id, params={'format': format})
        return self.service.decode(schema, resp)

    def get_plaintext(self, id): # pylint: disable=invalid-name,redefined-builtin
        """Get a config as plaintext."""
        return self.service.get_id(self.base, id, params={'format': 'text'})

    def create(self, resource):
        """Create a new config."""
        schema = ConfigSchema(exclude=('id', 'created', 'updated', 'result_id'))
        json = self.service.encode(schema, resource)

        schema = ConfigSchema()
        resp = self.service.create(self.base, json)
        return self.service.decode(schema, resp)

    def edit(self, resource):
        """Edit a config."""
        schema = ConfigSchema(exclude=('id', 'created', 'updated', 'result_id'))
        json = self.service.encode(schema, resource)

        schema = ConfigSchema()
        resp = self.service.edit(self.base, resource.id, json)
        return self.service.decode(schema, resp)

    def delete(self, id): # pylint: disable=invalid-name,redefined-builtin
        """Delete a config."""
        return self.service.delete_id(self.base, id)

    def get_shares(self, id): # pylint: disable=invalid-name,redefined-builtin
        """Get shares for a config."""
        return self.service.get_shares(self.base, id)

    def edit_shares(self, id, user_ids): # pylint: disable=invalid-name,redefined-builtin
        """Edit shares for a config."""
        return self.service.edit_shares(self.base, id, user_ids)

    def export(self, id): # pylint: disable=invalid-name,redefined-builtin
        """Export a config."""
        return self.service.export(self.base, id)

    def check_config(self, contents):
        """Process config contents with cdrouter-cli -check-config."""
        schema = CheckConfigSchema()
        resp = self.service.post(self.base,
                                 params={'process': 'check'}, json={'contents': contents})
        return self.service.decode(schema, resp)

    def upgrade_config(self, contents):
        """Process config contents with cdrouter-cli -upgrade-config."""
        schema = UpgradeConfigSchema()
        resp = self.service.post(self.base,
                                 params={'process': 'upgrade'}, json={'contents': contents})
        return self.service.decode(schema, resp)

    def get_networks(self, contents):
        """Process config contents with cdrouter-cli -print-networks-json."""
        schema = NetworksSchema()
        resp = self.service.post(self.base,
                                 params={'process': 'networks'}, json={'contents': contents})
        return self.service.decode(schema, resp)

    def bulk_export(self, ids):
        """Bulk export a set of configs."""
        return self.service.bulk_export(self.base, ids)

    def bulk_copy(self, ids):
        """Bulk copy a set of configs."""
        schema = ConfigSchema()
        return self.service.bulk_copy(self.base, self.RESOURCE, ids, schema)

    def bulk_edit(self, _fields, ids=None, filter=None, all=False, testvars=None): # pylint: disable=redefined-builtin
        """Bulk edit a set of configs."""
        return self.service.bulk_edit(self.base, self.RESOURCE,
                                      _fields, ids=ids, filter=filter, all=all, testvars=testvars)

    def bulk_delete(self, ids=None, filter=None, all=False): # pylint: disable=redefined-builtin
        """Bulk delete a set of configs."""
        return self.service.bulk_delete(self.base, self.RESOURCE,
                                        ids=ids, filter=filter, all=all)

    def list_testvars(self, id): # pylint: disable=invalid-name,redefined-builtin
        """Get a list of a config's testvars."""
        schema = TestvarSchema()
        resp = self.service.get(self.base+str(id)+'/testvars/')
        return self.service.decode(schema, resp, many=True)

    def get_testvar(self, id, name, group=None): # pylint: disable=invalid-name,redefined-builtin
        """Get a testvar from a config."""
        schema = TestvarSchema()
        resp = self.service.get(self.base+str(id)+'/testvars/'+name+'/', params={'group': group})
        return self.service.decode(schema, resp)

    def edit_testvar(self, id, resource): # pylint: disable=invalid-name,redefined-builtin
        """Edit a testvar in a config."""
        schema = TestvarSchema()
        json = self.service.encode(schema, resource)

        schema = TestvarSchema()
        resp = self.service.patch(self.base+str(id)+'/testvars/'+resource.name+'/',
                                  params={'group': resource.group}, json=json)
        return self.service.decode(schema, resp)

    def delete_testvar(self, id, name, group=None): # pylint: disable=invalid-name,redefined-builtin
        """Delete a testvar in a config. Deleting a testvar unsets any
        explicitly configured value for it in the config.
        """
        return self.service.delete(self.base+str(id)+'/testvars/'+name+'/', params={'group': group})

    def bulk_edit_testvars(self, id, testvars): # pylint: disable=invalid-name,redefined-builtin
        """Bulk edit a config's testvars."""
        schema = TestvarSchema()
        json = self.service.encode(schema, testvars, many=True)

        schema = TestvarSchema()
        resp = self.service.post(self.base+str(id)+'/testvars/',
                                 params={'bulk': 'edit'}, json=json)
        return self.service.decode(schema, resp, many=True)



#
# Copyright (c) 2017 by QA Cafe.
# All Rights Reserved.
#

"""Module for accessing CDRouter Users."""

from marshmallow import Schema, fields, post_load
from .cdr_datetime import DateTime

class User(object):
    def __init__(self, **kwargs):
        self.id = kwargs.get('id', None)
        self.admin = kwargs.get('admin', None)
        self.disabled = kwargs.get('disabled', None)
        self.name = kwargs.get('name', None)
        self.description = kwargs.get('description', None)
        self.created = kwargs.get('created', None)
        self.updated = kwargs.get('updated', None)
        self.token = kwargs.get('token', None)

        # only needed for change_password
        self.password = kwargs.get('password', None)
        self.password_confirm = kwargs.get('password_confirm', None)

class UserSchema(Schema):
    id = fields.Str()
    admin = fields.Bool()
    disabled = fields.Bool()
    name = fields.Str()
    description = fields.Str()
    created = DateTime()
    updated = DateTime()
    token = fields.Str()

    password = fields.Str()
    password_confirm = fields.Str()

    @post_load
    def post_load(self, data):
        return User(**data)

class UsersService(object):
    """Service for accessing CDRouter Users."""

    RESOURCE = 'users'
    BASE = RESOURCE + '/'

    def __init__(self, service):
        self.service = service
        self.base = self.BASE

    def list(self, filter=None, sort=None, limit=None, page=None): # pylint: disable=redefined-builtin
        """Get a list of users."""
        schema = UserSchema(exclude=('created', 'updated', 'token', 'password', 'password_confirm'))
        resp = self.service.list(self.base, filter, sort, limit, page)
        return self.service.decode(schema, resp, many=True)

    def get(self, id): # pylint: disable=invalid-name,redefined-builtin
        """Get a user."""
        schema = UserSchema()
        resp = self.service.get_id(self.base, id)
        return self.service.decode(schema, resp)

    def create(self, resource):
        """Create a new user."""
        schema = UserSchema(exclude=('id', 'created', 'updated', 'token'))
        json = self.service.encode(schema, resource)

        schema = UserSchema(exclude=('password', 'password_confirm'))
        resp = self.service.create(self.base, json)
        return self.service.decode(schema, resp)

    def edit(self, resource):
        """Edit a user."""
        schema = UserSchema(exclude=('id', 'created', 'updated', 'token', 'password', 'password_confirm'))
        json = self.service.encode(schema, resource)

        schema = UserSchema(exclude=('password', 'password_confirm'))
        resp = self.service.edit(self.base, resource.id, json)
        return self.service.decode(schema, resp)

    def change_password(self, id, new, old=None, change_token=True): # pylint: disable=invalid-name,redefined-builtin
        """Change a user's password."""
        schema = UserSchema(exclude=('password', 'password_confirm'))
        resp = self.service.post(self.base+str(id)+'/password/',
                                  params={'change_token', change_token},
                                  json={'old': old, 'new': new, 'new_confirm': new})
        return self.service.decode(schema, resp)

    def change_token(self, id): # pylint: disable=invalid-name,redefined-builtin
        """Change a user's token."""
        schema = UserSchema(exclude=('password', 'password_confirm'))
        resp = self.service.post(self.base+str(id)+'/token/')
        return self.service.decode(schema, resp)

    def delete(self, id): # pylint: disable=invalid-name,redefined-builtin
        """Delete a user."""
        return self.service.delete_id(self.base, id)

    def bulk_copy(self, ids):
        """Bulk copy a set of users."""
        schema = UserSchema()
        return self.service.bulk_copy(self.base, self.RESOURCE, ids, schema)

    def bulk_edit(self, _fields, ids=None, filter=None, all=False): # pylint: disable=redefined-builtin
        """Bulk edit a set of users."""
        return self.service.bulk_edit(self.base, self.RESOURCE, _fields, ids=ids, filter=filter, all=all)

    def bulk_delete(self, ids=None, filter=None, all=False): # pylint: disable=redefined-builtin
        """Bulk delete a set of users."""
        return self.service.bulk_delete(self.base, self.RESOURCE, ids=ids, filter=filter, all=all)



#
# Copyright (c) 2017 by QA Cafe.
# All Rights Reserved.
#

"""Module for accessing CDRouter Captures."""

from marshmallow import Schema, fields, post_load

class Section(object):
    def __init__(self, **kwargs):
        self.value = kwargs.get('value', None)

class SectionSchema(Schema):
    value = fields.Str()

    @post_load
    def post_load(self, data):
        return Section(**data)

class Structure(object):
    def __init__(self, **kwargs):
        self.sections = kwargs.get('sections', None)

class StructureSchema(Schema):
    sections = fields.Nested(SectionSchema, many=True)

    @post_load
    def post_load(self, data):
        return Structure(**data)

class SummaryPacket(object):
    def __init__(self, **kwargs):
        self.sections = kwargs.get('sections', None)

class SummaryPacketSchema(Schema):
    sections = fields.Nested(SectionSchema, many=True)

    @post_load
    def post_load(self, data):
        return SummaryPacket(**data)

class Summary(object):
    def __init__(self, **kwargs):
        self.structure = kwargs.get('structure', None)
        self.summaries = kwargs.get('summaries', None)

class SummarySchema(Schema):
    structure = fields.Nested(StructureSchema)
    summaries = fields.Nested(SummaryPacketSchema, many=True)

    @post_load
    def post_load(self, data):
        return Summary(**data)

class Field(object):
    def __init__(self, **kwargs):
        self.name = kwargs.get('name', None)
        self.show_name = kwargs.get('show_name', None)
        self.hide = kwargs.get('hide', None)
        self.size = kwargs.get('size', None)
        self.pos = kwargs.get('pos', None)
        self.show = kwargs.get('show', None)
        self.fields = kwargs.get('fields', None)
        self.protos = kwargs.get('protos', None)

class FieldSchema(Schema):
    name = fields.Str()
    show_name = fields.Str()
    hide = fields.Str()
    size = fields.Str()
    pos = fields.Str()
    show = fields.Str()
    _fields = fields.Nested('self', attribute='fields', load_from='fields', dump_to='fields', many=True)
    protos = fields.Nested('ProtoSchema', many=True)

    @post_load
    def post_load(self, data):
        return Field(**data)

class Proto(object):
    def __init__(self, **kwargs):
        self.name = kwargs.get('name', None)
        self.pos = kwargs.get('pos', None)
        self.show = kwargs.get('show', None)
        self.show_name = kwargs.get('show_name', None)
        self.value = kwargs.get('value', None)
        self.size = kwargs.get('size', None)
        self.fields = kwargs.get('fields', None)
        self.protos = kwargs.get('protos', None)

class ProtoSchema(Schema):
    name = fields.Str()
    pos = fields.Str()
    show = fields.Str()
    show_name = fields.Str()
    value = fields.Str()
    size = fields.Str()
    # _fields = fields.Nested(FieldSchema, attribute='fields', load_from='fields', dump_to='fields', many=True)
    # protos = fields.Nested('self', many=True)

    @post_load
    def post_load(self, data):
        return Proto(**data)

class Packet(object):
    def __init__(self, **kwargs):
        self.protos = kwargs.get('protos', None)

class PacketSchema(Schema):
    protos = fields.Nested(ProtoSchema, many=True)

    @post_load
    def post_load(self, data):
        return Packet(**data)

class Decode(object):
    def __init__(self, **kwargs):
        self.packets = kwargs.get('packets', None)

class DecodeSchema(Schema):
    packets = fields.Nested(PacketSchema, many=True)

    @post_load
    def post_load(self, data):
        return Decode(**data)

class ASCIIByte(object):
    def __init__(self, **kwargs):
        self.byte = kwargs.get('byte', None)
        self.pos = kwargs.get('pos', None)

class ASCIIByteSchema(Schema):
    byte = fields.Str()
    pos = fields.Int()

    @post_load
    def post_load(self, data):
        return ASCIIByte(**data)

class ASCIILine(object):
    def __init__(self, **kwargs):
        self.raw = kwargs.get('raw', None)
        self.offset = kwargs.get('offset', None)
        self.ascii = kwargs.get('ascii', None)
        self.hex = kwargs.get('hex', None)

class ASCIILineSchema(Schema):
    raw = fields.Str()
    offset = fields.Str()
    ascii = fields.Nested(ASCIIByteSchema, many=True)
    hex = fields.Nested(ASCIIByteSchema, many=True)

    @post_load
    def post_load(self, data):
        return ASCIILine(**data)

class ASCIIFrame(object):
    def __init__(self, **kwargs):
        self.name = kwargs.get('name', None)
        self.lines = kwargs.get('lines', None)

class ASCIIFrameSchema(Schema):
    name = fields.Str(missing=None)
    lines = fields.Nested(ASCIILineSchema, many=True)

    @post_load
    def post_load(self, data):
        return ASCIIFrame(**data)

class ASCII(object):
    def __init__(self, **kwargs):
        self.frame = kwargs.get('frame', None)
        self.reassembled = kwargs.get('reassembled', None)

class ASCIISchema(Schema):
    frame = fields.Nested(ASCIIFrameSchema, missing=None)
    reassembled = fields.Nested(ASCIIFrameSchema, missing=None)

    @post_load
    def post_load(self, data):
        return ASCII(**data)

class Capture(object):
    def __init__(self, **kwargs):
        self.id = kwargs.get('id', None)
        self.seq = kwargs.get('seq', None)
        self.interface = kwargs.get('interface', None)
        self.filename = kwargs.get('filename', None)

class CaptureSchema(Schema):
    id = fields.Str()
    seq = fields.Str()
    interface = fields.Str()
    filename = fields.Str()

    @post_load
    def post_load(self, data):
        return Capture(**data)

class CloudShark(object):
    def __init__(self, **kwargs):
        self.url = kwargs.get('url', None)

class CloudSharkSchema(Schema):
    url = fields.Str()

    @post_load
    def post_load(self, data):
        return CloudShark(**data)

class CapturesService(object):
    """Service for accessing CDRouter Captures."""

    RESOURCE = 'captures'
    BASE = RESOURCE + '/'

    def __init__(self, service):
        self.service = service

    def _base(self, id, seq): # pylint: disable=invalid-name,redefined-builtin
        return 'results/'+str(id)+'/tests/'+str(seq)+self.BASE

    def list(self, id, seq): # pylint: disable=invalid-name,redefined-builtin
        """Get a list of captures."""
        schema = CaptureSchema(exclude=('id', 'seq'))
        resp = self.service.list(self._base(id, seq), filter)
        return self.service.decode(schema, resp, many=True)

    def get(self, id, seq, intf): # pylint: disable=invalid-name,redefined-builtin
        """Get a capture."""
        schema = CaptureSchema()
        resp = self.service.get_id(self._base(id, seq), intf)
        return self.service.decode(schema, resp)

    def download(self, id, seq, intf, inline=False): # pylint: disable=invalid-name,redefined-builtin
        """Download a capture as a PCAP file."""
        return self.service.get_id(self._base(id, seq), intf, params={'format': format, 'inline': inline})

    def summary(self, id, seq, intf, filter=None, inline=False): # pylint: disable=invalid-name,redefined-builtin
        """Get a capture's summary."""
        schema = SummarySchema()
        resp = self.service.get(self._base(id, seq)+str(intf)+'/summary/',
                                params={'filter': filter, 'inline': inline})
        return self.service.decode(schema, resp)

    def decode(self, id, seq, intf, filter=None, frame=None, inline=False): # pylint: disable=invalid-name,redefined-builtin
        """Get a capture's decode."""
        schema = DecodeSchema()
        resp = self.service.get(self._base(id, seq)+str(intf)+'/decode/',
                                params={'filter': filter, 'frame': frame, 'inline': inline})
        return self.service.decode(schema, resp)

    def ascii(self, id, seq, intf, filter=None, frame=None, inline=False): # pylint: disable=invalid-name,redefined-builtin
        """Get a capture's ASCII (hex dump)."""
        schema = ASCIISchema()
        resp = self.service.get(self._base(id, seq)+str(intf)+'/ascii/',
                                params={'filter': filter, 'frame': frame, 'inline': inline})
        return self.service.decode(schema, resp)

    def send_to_cloudshark(self, id, seq, intf, inline=False): # pylint: disable=invalid-name,redefined-builtin
        """Send a capture to a CloudShark Appliance. Both
        cloudshark_appliance_url and cloudshark_appliance_token must
        be properly configured via system preferences.
        """
        schema = CloudSharkSchema()
        resp = self.service.post(self._base(id, seq)+str(intf)+'/cloudshark/', params={'inline': inline})
        return self.service.decode(schema, resp)



#
# Copyright (c) 2017 by QA Cafe.
# All Rights Reserved.
#

"""Module for accessing CDRouter Jobs."""

from marshmallow import Schema, fields, post_load
from .cdr_datetime import DateTime

class Job(object):
    def __init__(self, **kwargs):
        self.id = kwargs.get('id', None)
        self.status = kwargs.get('status', None)
        self.options = kwargs.get('options', None)
        self.package_id = kwargs.get('package_id', None)
        self.package_name = kwargs.get('package_name', None)
        self.device_id = kwargs.get('device_id', None)
        self.device_name = kwargs.get('device_name', None)
        self.result_id = kwargs.get('result_id', None)
        self.user_id = kwargs.get('user_id', None)
        self.created = kwargs.get('created', None)
        self.updated = kwargs.get('updated', None)

class Options(object):
    def __init__(self, **kwargs):
        self.tags = kwargs.get('tags', None)
        self.skip_tests = kwargs.get('skip_tests', None)
        self.begin_at = kwargs.get('begin_at', None)
        self.end_at = kwargs.get('end_at', None)
        self.extra_cli_args = kwargs.get('extra_cli_args', None)

class OptionsSchema(Schema):
    tags = fields.List(fields.Str(), missing=None)
    skip_tests = fields.List(fields.Str(), missing=None)
    begin_at = fields.Str()
    end_at = fields.Str()
    extra_cli_args = fields.Str()

    @post_load
    def post_load(self, data):
        return Options(**data)

class JobSchema(Schema):
    id = fields.Str()
    status = fields.Str()
    options = fields.Nested(OptionsSchema)
    package_id = fields.Str()
    package_name = fields.Str()
    device_id = fields.Str()
    device_name = fields.Str()
    result_id = fields.Str(missing=None)
    user_id = fields.Str()
    created = DateTime()
    updated = DateTime()

    @post_load
    def post_load(self, data):
        return Job(**data)

class JobsService(object):
    """Service for accessing CDRouter Jobs."""

    RESOURCE = 'jobs'
    BASE = RESOURCE + '/'

    def __init__(self, service):
        self.service = service
        self.base = self.BASE

    def list(self, filter=None, sort=None, limit=None, page=None): # pylint: disable=redefined-builtin
        """Get a list of jobs."""
        schema = JobSchema()
        resp = self.service.list(self.base, filter, sort, limit, page)
        return self.service.decode(schema, resp, many=True)

    def get(self, id): # pylint: disable=invalid-name,redefined-builtin
        """Get a job."""
        schema = JobSchema()
        resp = self.service.get_id(self.base, id)
        return self.service.decode(schema, resp)

    def edit(self, resource):
        """Edit a tag."""
        schema = JobSchema(exclude=('id', 'status', 'options', 'package_name', 'device_name', 'result_id', 'user_id', 'created', 'updated'))
        json = self.service.encode(schema, resource)

        schema = JobSchema()
        resp = self.service.edit(self.base, resource.name, json)
        return self.service.decode(schema, resp)

    def launch(self, resource):
        """Launch a new job."""
        schema = JobSchema(exclude=('id', 'status', 'package_name', 'device_name', 'result_id', 'user_id', 'created', 'updated'))
        json = self.service.encode(schema, resource)

        schema = JobSchema()
        resp = self.service.create(self.base, json)
        return self.service.decode(schema, resp)

    def delete(self, id): # pylint: disable=invalid-name,redefined-builtin
        """Delete a job."""
        return self.service.delete_id(self.base, id)

    @staticmethod
    def _package_id(id): # pylint: disable=invalid-name,redefined-builtin
        if isinstance(id, (str, int)):
            return {'package_id': str(id)}
        return id

    def bulk_launch(self, jobs=None, filter=None, all=False): # pylint: disable=redefined-builtin
        """Bulk launch a set of jobs."""
        json = None
        if jobs is not None:
            schema = JobSchema(exclude=('id', 'status', 'package_name', 'device_name', 'result_id', 'user_id', 'created', 'updated'))
            jobs_json = self.service.encode(schema, jobs, many=True)
            json = {self.RESOURCE: jobs_json}

        schema = JobSchema()
        resp = self.service.post(self.base,
                                 params={'bulk': 'launch', 'filter': filter, 'all': all}, json=json)
        return self.service.decode(schema, resp, many=True)

    def bulk_delete(self, ids=None, filter=None, all=False): # pylint: disable=redefined-builtin
        """Bulk delete a set of jobs."""
        return self.service.bulk_delete(self.base, self.RESOURCE, ids=ids, filter=filter, all=all)



#
# Copyright (c) 2017 by QA Cafe.
# All Rights Reserved.
#

"""Module for accessing CDRouter Imports."""

import os.path
from marshmallow import Schema, fields, post_load
from .cdr_datetime import DateTime

class Import(object):
    def __init__(self, **kwargs):
        self.id = kwargs.get('id', None)
        self.user_id = kwargs.get('user_id', None)
        self.created = kwargs.get('created', None)
        self.updated = kwargs.get('updated', None)
        self.archive = kwargs.get('archive', None)
        self.path = kwargs.get('path', None)
        self.url = kwargs.get('url', None)
        self.insecure = kwargs.get('insecure', None)
        self.size = kwargs.get('size', None)

class ImportSchema(Schema):
    id = fields.Str()
    user_id = fields.Str()
    created = DateTime()
    updated = DateTime()
    archive = fields.Str()
    path = fields.Str()
    url = fields.Str()
    insecure = fields.Bool()
    size = fields.Int()

    @post_load
    def post_load(self, data):
        return Import(**data)

class Response(object):
    def __init__(self, **kwargs):
        self.imported = kwargs.get('imported', None)
        self.id = kwargs.get('id', None)
        self.name = kwargs.get('name', None)
        self.message = kwargs.get('message', None)

class ResponseSchema(Schema):
    imported = fields.Bool()
    id = fields.Str(missing=None)
    name = fields.Str(missing=None)
    message = fields.Str(missing=None)

    @post_load
    def post_load(self, data):
        return Response(**data)

class Request(object):
    def __init__(self, **kwargs):
        self.replace_existing = kwargs.get('replace_existing', None)

        self.configs = kwargs.get('configs', None)
        self.devices = kwargs.get('devices', None)
        self.packages = kwargs.get('packages', None)
        self.results = kwargs.get('results', None)

        self.tags = kwargs.get('tags', None)

class RequestSchema(Schema):
    replace_existing = fields.Bool()

    configs = fields.Dict()
    devices = fields.Dict()
    packages = fields.Dict()
    results = fields.Dict()

    tags = fields.List(fields.Str(), missing=None)

    @post_load
    def post_load(self, data):
        return Request(**data)

class ImportsService(object):
    """Service for accessing CDRouter Imports."""

    RESOURCE = 'imports'
    BASE = RESOURCE + '/'

    def __init__(self, service):
        self.service = service
        self.base = self.BASE

    def list(self):
        """Get a list of staged (in-progress) imports."""
        schema = ImportSchema()
        resp = self.service.list(self.base)
        return self.service.decode(schema, resp, many=True)

    def stage_import_from_file(self, filepath):
        """Stage an import from a file upload."""
        schema = ImportSchema()
        resp = self.service.post(self.base,
                                 files={'file': (os.path.basename(filepath), open(filepath, 'rb'))})
        return self.service.decode(schema, resp)

    def stage_import_from_filesystem(self, filepath):
        """Stage an import from a filesystem path."""
        schema = ImportSchema()
        resp = self.service.post(self.base,
                                 params={'path': filepath})
        return self.service.decode(schema, resp)

    def stage_import_from_url(self, url, token=None, insecure=False):
        """Stage an import from a URL to another CDRouter system."""
        schema = ImportSchema()
        resp = self.service.post(self.base,
                                 params={'url': url, 'token': token, 'insecure': insecure})
        return self.service.decode(schema, resp)

    def get(self, id): # pylint: disable=invalid-name,redefined-builtin
        """Get a staged import."""
        schema = ImportSchema()
        resp = self.service.get_id(self.base, id)
        return self.service.decode(schema, resp)

    def get_commit_request(self, id): # pylint: disable=invalid-name,redefined-builtin
        """Get a commit request for a staged import."""
        schema = RequestSchema()
        resp = self.service.get(self.base+str(id)+'/request/')
        return self.service.decode(schema, resp)

    def commit(self, id, impreq): # pylint: disable=invalid-name,redefined-builtin
        """Commit a staged import."""
        schema = RequestSchema()
        json = self.service.encode(schema, impreq)

        schema = RequestSchema()
        resp = self.service.post(self.base+str(id)+'/', json=json)
        return self.service.decode(schema, resp)

    def delete(self, id): # pylint: disable=invalid-name,redefined-builtin
        """Delete a staged import."""
        return self.service.delete_id(self.base, id)



#
# Copyright (c) 2017 by QA Cafe.
# All Rights Reserved.
#

"""Module for accessing CDRouter Tags."""

from marshmallow import Schema, fields, post_load

class ResourceTags(object):
    def __init__(self, **kwargs):
        self.id = kwargs.get('id', None)
        self.name = kwargs.get('name', None)
        self.tags = kwargs.get('tags', None)

class Tag(object):
    def __init__(self, **kwargs):
        self.resource = kwargs.get('resource', None)
        self.id = kwargs.get('id', None)
        self.name = kwargs.get('name', None)
        self.count = kwargs.get('count', None)
        self.tags = kwargs.get('tags', None)

        self.configs = kwargs.get('configs', None)
        self.devices = kwargs.get('devices', None)
        self.packages = kwargs.get('packages', None)
        self.results = kwargs.get('results', None)

class ResourceTagsSchema(Schema):
    id = fields.Str()
    name = fields.Str()
    tags = fields.List(fields.Str())

    @post_load
    def post_load(self, data):
        return ResourceTags(**data)

class TagSchema(Schema):
    resource = fields.Str()
    id = fields.Str()
    name = fields.Str()
    count = fields.Int()
    tags = fields.List(fields.Str())

    configs = fields.Nested(ResourceTagsSchema, many=True)
    devices = fields.Nested(ResourceTagsSchema, many=True)
    packages = fields.Nested(ResourceTagsSchema, many=True)
    results = fields.Nested(ResourceTagsSchema, many=True)

    @post_load
    def post_load(self, data):
        return Tag(**data)

class TagsService(object):
    """Service for accessing CDRouter Tags."""

    RESOURCE = 'tags'
    BASE = RESOURCE + '/'

    def __init__(self, service):
        self.service = service
        self.base = self.BASE

    def list(self, resource=None, sort=None):
        """Get a list of tags."""
        schema = TagSchema()
        resp = self.service.get(self.base, params={'resource': resource, 'sort': sort})
        return self.service.decode(schema, resp, many=True)

    def get(self, name):
        """Get a tag."""
        schema = TagSchema()
        resp = self.service.get_id(self.base, name)
        return self.service.decode(schema, resp)

    def edit(self, resource):
        """Edit a tag."""
        schema = TagSchema(only=('name', 'configs', 'devices', 'packages', 'results'))
        json = self.service.encode(schema, resource)

        schema = TagSchema()
        resp = self.service.edit(self.base, resource.name, json)
        return self.service.decode(schema, resp)

    def delete(self, name):
        """Delete a tag."""
        return self.service.delete_id(self.base, name)



#
# Copyright (c) 2017 by QA Cafe.
# All Rights Reserved.
#

"""Module for accessing CDRouter Exports."""

import io

from requests_toolbelt.downloadutils import stream

class ExportsService(object):
    """Service for accessing CDRouter Exports."""

    RESOURCE = 'exports'
    BASE = RESOURCE + '/'

    def __init__(self, service):
        self.service = service
        self.base = self.BASE

    def bulk_export(self, config_ids=None, device_ids=None, package_ids=None, result_ids=None, exclude_captures=False):
        """Bulk export a set of configs, devices, packages and results."""
        if config_ids is None:
            config_ids = []
        if device_ids is None:
            device_ids = []
        if package_ids is None:
            package_ids = []
        if result_ids is None:
            result_ids = []
        json = {
            'configs': map(int, config_ids),
            'devices': map(int, device_ids),
            'packages': map(int, package_ids),
            'results': map(int, result_ids),
            'options': {'exclude_captures': exclude_captures}
        }
        resp = self.service.post(self.base, json=json)
        resp.raise_for_status()
        b = io.BytesIO()
        stream.stream_response_to_file(resp, path=b)
        return (b, self.service.filename(resp))



#
# Copyright (c) 2017 by QA Cafe.
# All Rights Reserved.
#

"""Module for accessing CDRouter History."""

from marshmallow import Schema, fields, post_load
from .cdr_datetime import DateTime

class History(object):
    def __init__(self, **kwargs):
        self.user_id = kwargs.get('user_id', None)
        self.created = kwargs.get('created', None)
        self.resource = kwargs.get('resource', None)
        self.id = kwargs.get('id', None)
        self.name = kwargs.get('name', None)
        self.action = kwargs.get('action', None)
        self.description = kwargs.get('description', None)

class HistorySchema(Schema):
    user_id = fields.Str()
    created = DateTime()
    resource = fields.Str()
    id = fields.Str()
    name = fields.Str()
    action = fields.Str()
    description = fields.Str()

    @post_load
    def post_load(self, data):
        return History(**data)

class HistoryService(object):
    """Service for accessing CDRouter History."""

    RESOURCE = 'history'
    BASE = RESOURCE + '/'

    def __init__(self, service):
        self.service = service
        self.base = self.BASE

    def list(self, filter=None, sort=None, limit=None, page=None): # pylint: disable=redefined-builtin
        """Get a list of history entries."""
        schema = HistorySchema()
        resp = self.service.list(self.base, filter, sort, limit, page)
        return self.service.decode(schema, resp, many=True)



#
# Copyright (c) 2017 by QA Cafe.
# All Rights Reserved.
#

"""Module for accessing CDRouter Devices."""

from marshmallow import Schema, fields, post_load
from .cdr_datetime import DateTime

class Device(object):
    def __init__(self, **kwargs):
        self.id = kwargs.get('id', None)
        self.name = kwargs.get('name', None)
        self.created = kwargs.get('created', None)
        self.updated = kwargs.get('updated', None)
        self.user_id = kwargs.get('user_id', None)
        self.result_id = kwargs.get('result_id', None)
        self.attachments_dir = kwargs.get('attachments_dir', None)
        self.picture_id = kwargs.get('picture_id', None)
        self.tags = kwargs.get('tags', None)

        self.default_ip = kwargs.get('default_ip', None)
        self.default_login = kwargs.get('default_login', None)
        self.default_password = kwargs.get('default_password', None)
        self.location = kwargs.get('location', None)

        self.device_category = kwargs.get('device_category', None)
        self.manufacturer = kwargs.get('manufacturer', None)
        self.manufacturer_oui = kwargs.get('manufacturer_oui', None)
        self.model_name = kwargs.get('model_name', None)
        self.model_number = kwargs.get('model_number', None)
        self.description = kwargs.get('description', None)
        self.product_class = kwargs.get('product_class', None)
        self.serial_number = kwargs.get('serial_number', None)
        self.hardware_version = kwargs.get('hardware_version', None)
        self.software_version = kwargs.get('software_version', None)
        self.provisioning_code = kwargs.get('provisioning_code', None)

        self.note = kwargs.get('note', None)

class DeviceSchema(Schema):
    id = fields.Str()
    name = fields.Str()
    created = DateTime()
    updated = DateTime()
    user_id = fields.Str()
    result_id = fields.Str(missing=None)
    attachments_dir = fields.Str(missing=None)
    picture_id = fields.Str()
    tags = fields.List(fields.Str())

    default_ip = fields.Str()
    default_login = fields.Str()
    default_password = fields.Str()
    location = fields.Str()

    device_category = fields.Str()
    manufacturer = fields.Str()
    manufacturer_oui = fields.Str()
    model_name = fields.Str()
    model_number = fields.Str()
    description = fields.Str()
    product_class = fields.Str()
    serial_number = fields.Str()
    hardware_version = fields.Str()
    software_version = fields.Str()
    provisioning_code = fields.Str()

    note = fields.Str()

    @post_load
    def post_load(self, data):
        return Device(**data)

class DevicesService(object):
    """Service for accessing CDRouter Devices."""

    RESOURCE = 'devices'
    BASE = RESOURCE + '/'

    def __init__(self, service):
        self.service = service
        self.base = self.BASE

    def list(self, filter=None, sort=None, limit=None, page=None): # pylint: disable=redefined-builtin
        """Get a list of devices."""
        schema = DeviceSchema(exclude=('attachments_dir', 'default_ip', 'default_login', 'default_password',
                                       'location', 'device_category', 'manufacturer', 'manufacturer_oui',
                                       'model_name', 'model_number', 'product_class', 'serial_number',
                                       'hardware_version', 'software_version', 'provisioning_code', 'note'))
        resp = self.service.list(self.base, filter, sort, limit, page)
        return self.service.decode(schema, resp, many=True)

    def get(self, id): # pylint: disable=invalid-name,redefined-builtin
        """Get a device."""
        schema = DeviceSchema()
        resp = self.service.get(self.base, id)
        return self.service.decode(schema, resp)

    def create(self, resource):
        """Create a new device."""
        schema = DeviceSchema(exclude=('id', 'created', 'updated', 'result_id', 'attachments_dir'))
        json = self.service.encode(schema, resource)

        schema = DeviceSchema()
        resp = self.service.create(self.base, json)
        return self.service.decode(schema, resp)

    def edit(self, resource):
        """Edit a device."""
        schema = DeviceSchema(exclude=('id', 'created', 'updated', 'result_id', 'attachments_dir'))
        json = self.service.encode(schema, resource)

        schema = DeviceSchema()
        resp = self.service.edit(self.base, resource.id, json)
        return self.service.decode(schema, resp)

    def delete(self, id): # pylint: disable=invalid-name,redefined-builtin
        """Delete a device."""
        return self.service.delete_id(self.base, id)

    def get_shares(self, id): # pylint: disable=invalid-name,redefined-builtin
        """Get shares for a device."""
        return self.service.get_shares(self.base, id)

    def edit_shares(self, id, user_ids): # pylint: disable=invalid-name,redefined-builtin
        """Edit shares for a device."""
        return self.service.edit_shares(self.base, id, user_ids)

    def export(self, id): # pylint: disable=invalid-name,redefined-builtin
        """Export a device."""
        return self.service.export(self.base, id)

    def bulk_export(self, ids):
        """Bulk export a set of devices."""
        return self.service.bulk_export(self.base, ids)

    def bulk_copy(self, ids):
        """Bulk copy a set of devices."""
        schema = DeviceSchema()
        return self.service.bulk_copy(self.base, self.RESOURCE, ids, schema)

    def bulk_edit(self, _fields, ids=None, filter=None, all=False): # pylint: disable=redefined-builtin
        """Bulk edit a set of devices."""
        return self.service.bulk_edit(self.base, self.RESOURCE, _fields, ids=ids,
                                      filter=filter, all=all)

    def bulk_delete(self, ids=None, filter=None, all=False): # pylint: disable=redefined-builtin
        """Bulk delete a set of devices."""
        return self.service.bulk_delete(self.base, self.RESOURCE, ids=ids,
                                        filter=filter, all=all)



#
# Copyright (c) 2017 by QA Cafe.
# All Rights Reserved.
#

"""Module for accessing CDRouter Packages."""

from marshmallow import Schema, fields, post_load
from .testsuites import TestSchema
from .cdr_datetime import DateTime

class Analyze(object):
    def __init__(self, **kwargs):
        self.total_count = kwargs.get('total_count', None)
        self.run_count = kwargs.get('run_count', None)
        self.skipped_count = kwargs.get('skipped_count', None)
        self.skipped_tests = kwargs.get('skipped_tests', None)

class AnalyzeSchema(Schema):
    total_count = fields.Int()
    run_count = fields.Int()
    skipped_count = fields.Int()
    skipped_tests = fields.Nested(TestSchema, many=True)

    @post_load
    def post_load(self, data):
        return Analyze(**data)

class Options(object):
    def __init__(self, **kwargs):
        self.forever = kwargs.get('forever', None)
        self.loop = kwargs.get('loop', None)
        self.repeat = kwargs.get('repeat', None)
        self.maxfail = kwargs.get('maxfail', None)
        self.duration = kwargs.get('duration', None)
        self.wait = kwargs.get('wait', None)
        self.pause = kwargs.get('pause', None)
        self.shuffle = kwargs.get('shuffle', None)
        self.seed = kwargs.get('seed', None)
        self.retry = kwargs.get('retry', None)
        self.rdelay = kwargs.get('rdelay', None)
        self.sync = kwargs.get('sync', None)

class OptionsSchema(Schema):
    forever = fields.Bool()
    loop = fields.Str()
    repeat = fields.Str()
    maxfail = fields.Str()
    duration = fields.Str()
    wait = fields.Str()
    pause = fields.Bool()
    shuffle = fields.Bool()
    seed = fields.Str()
    retry = fields.Str()
    rdelay = fields.Str()
    sync = fields.Bool()

    @post_load
    def post_load(self, data):
        return Options(**data)

class Package(object):
    def __init__(self, **kwargs):
        self.id = kwargs.get('id', None)
        self.name = kwargs.get('name', None)
        self.description = kwargs.get('description', None)
        self.created = kwargs.get('created', None)
        self.updated = kwargs.get('updated', None)
        self.test_count = kwargs.get('test_count', None)
        self.testlist = kwargs.get('testlist', None)
        self.extra_cli_args = kwargs.get('extra_cli_args', None)
        self.user_id = kwargs.get('user_id', None)
        self.agent_id = kwargs.get('agent_id', None)
        self.config_id = kwargs.get('config_id', None)
        self.result_id = kwargs.get('result_id', None)
        self.device_id = kwargs.get('device_id', None)
        self.options = kwargs.get('options', None)
        self.tags = kwargs.get('tags', None)
        self.use_as_testlist = kwargs.get('use_as_testlist', None)
        self.note = kwargs.get('note', None)

class PackageSchema(Schema):
    id = fields.Str()
    name = fields.Str()
    description = fields.Str()
    created = DateTime()
    updated = DateTime()
    test_count = fields.Str()
    testlist = fields.List(fields.Str())
    extra_cli_args = fields.Str()
    user_id = fields.Str()
    agent_id = fields.Str()
    config_id = fields.Str()
    result_id = fields.Str(missing=None)
    device_id = fields.Str()
    options = fields.Nested(OptionsSchema)
    tags = fields.List(fields.Str())
    use_as_testlist = fields.Bool()
    note = fields.Str(missing=None)

    @post_load
    def post_load(self, data):
        return Package(**data)

class PackagesService(object):
    """Service for accessing CDRouter Packages."""

    RESOURCE = 'packages'
    BASE = RESOURCE + '/'

    def __init__(self, service):
        self.service = service
        self.base = self.BASE

    def list(self, filter=None, sort=None, limit=None, page=None): # pylint: disable=redefined-builtin
        """Get a list of packages."""
        schema = PackageSchema(exclude=('testlist', 'extra_cli_args', 'agent_id', 'options', 'note'))
        resp = self.service.list(self.base, filter, sort, limit, page)
        return self.service.decode(schema, resp, many=True)

    def get(self, id): # pylint: disable=invalid-name,redefined-builtin
        """Get a package."""
        schema = PackageSchema()
        resp = self.service.get_id(self.base, id)
        return self.service.decode(schema, resp)

    def create(self, resource):
        """Create a new package."""
        schema = PackageSchema(exclude=('id', 'created', 'updated', 'test_count', 'agent_id', 'result_id'))
        json = self.service.encode(schema, resource)

        schema = PackageSchema()
        resp = self.service.create(self.base, json)
        return self.service.decode(schema, resp)

    def edit(self, resource):
        """Edit a package."""
        schema = PackageSchema(exclude=('id', 'created', 'updated', 'test_count', 'agent_id', 'result_id'))
        json = self.service.encode(schema, resource)

        schema = PackageSchema()
        resp = self.service.edit(self.base, resource.id, json)
        return self.service.decode(schema, resp)

    def delete(self, id): # pylint: disable=invalid-name,redefined-builtin
        """Delete a package."""
        return self.service.delete_id(self.base, id)

    def get_shares(self, id): # pylint: disable=invalid-name,redefined-builtin
        """Get shares for a package."""
        return self.service.get_shares(self.base, id)

    def edit_shares(self, id, user_ids): # pylint: disable=invalid-name,redefined-builtin
        """Edit shares for a package."""
        return self.service.edit_shares(self.base, id, user_ids)

    def export(self, id): # pylint: disable=invalid-name,redefined-builtin
        """Export a package."""
        return self.service.export(self.base, id)

    def analyze(self, id): # pylint: disable=invalid-name,redefined-builtin
        """Get a list of tests that will be skipped for a package."""
        schema = AnalyzeSchema()
        resp = self.service.post(self.base+str(id)+'/', params={'process': 'analyze'})
        return self.service.decode(schema, resp)

    def bulk_export(self, ids):
        """Bulk export a set of packages."""
        return self.service.bulk_export(self.base, ids)

    def bulk_copy(self, ids):
        """Bulk copy a set of packages."""
        schema = PackageSchema()
        return self.service.bulk_copy(self.base, self.RESOURCE, ids, schema)

    def bulk_edit(self, _fields, ids=None, filter=None, all=False): # pylint: disable=redefined-builtin
        """Bulk edit a set of packages."""
        return self.service.bulk_edit(self.base, self.RESOURCE,
                                      _fields, ids=ids, filter=filter, all=all)

    def bulk_delete(self, ids=None, filter=None, all=False): # pylint: disable=redefined-builtin
        """Bulk delete a set of packages."""
        return self.service.bulk_delete(self.base, self.RESOURCE, ids=ids, filter=filter, all=all)



#
# Copyright (c) 2017 by QA Cafe.
# All Rights Reserved.
#

"""Module for accessing CDRouter Testsuites."""

from marshmallow import Schema, fields, post_load

class Group(object):
    def __init__(self, **kwargs):
        self.id = kwargs.get('id', None)
        self.name = kwargs.get('name', None)
        self.index = kwargs.get('index', None)
        self.test_count = kwargs.get('test_count', None)
        self.modules = kwargs.get('modules', None)

class GroupSchema(Schema):
    id = fields.Int(missing=None)
    name = fields.Str()
    index = fields.Int()
    test_count = fields.Int()
    modules = fields.List(fields.Str())

    @post_load
    def post_load(self, data):
        return Group(**data)

class Module(object):
    def __init__(self, **kwargs):
        self.id = kwargs.get('id', None)
        self.name = kwargs.get('name', None)
        self.index = kwargs.get('index', None)
        self.group = kwargs.get('group', None)
        self.description = kwargs.get('description', None)
        self.test_count = kwargs.get('test_count', None)
        self.tests = kwargs.get('tests', None)
        self.labels = kwargs.get('labels', None)
        self.aliases = kwargs.get('aliases', None)

class ModuleSchema(Schema):
    id = fields.Int(missing=None)
    name = fields.Str()
    index = fields.Int()
    group = fields.Str()
    description = fields.Str()
    test_count = fields.Int()
    tests = fields.List(fields.Str())
    labels = fields.List(fields.Str())
    aliases = fields.List(fields.Str())

    @post_load
    def post_load(self, data):
        return Module(**data)

class Test(object):
    def __init__(self, **kwargs):
        self.id = kwargs.get('id', None)
        self.name = kwargs.get('name', None)
        self.index = kwargs.get('index', None)
        self.group = kwargs.get('group', None)
        self.module = kwargs.get('module', None)
        self.synopsis = kwargs.get('synopsis', None)
        self.description = kwargs.get('description', None)
        self.labels = kwargs.get('labels', None)
        self.aliases = kwargs.get('aliases', None)
        self.testvars = kwargs.get('testvars', None)

        self.skip_name = kwargs.get('skip_name', None)
        self.skip_reason = kwargs.get('skip_reason', None)

class TestSchema(Schema):
    id = fields.Int(missing=None)
    name = fields.Str()
    index = fields.Int()
    group = fields.Str()
    module = fields.Str()
    synopsis = fields.Str()
    description = fields.Str()
    labels = fields.List(fields.Str())
    aliases = fields.List(fields.Str())
    testvars = fields.List(fields.Str())

    skip_name = fields.Str(missing=None)
    skip_reason = fields.Str(missing=None)

    @post_load
    def post_load(self, data):
        return Test(**data)

class Label(object):
    def __init__(self, **kwargs):
        self.id = kwargs.get('id', None)
        self.name = kwargs.get('name', None)
        self.index = kwargs.get('index', None)
        self.reason = kwargs.get('reason', None)
        self.description = kwargs.get('description', None)
        self.modules = kwargs.get('modules', None)
        self.tests = kwargs.get('tests', None)

class LabelSchema(Schema):
    id = fields.Int(missing=None)
    name = fields.Str()
    index = fields.Int()
    reason = fields.Str()
    description = fields.Str()
    modules = fields.List(fields.Str())
    tests = fields.List(fields.Str())

    @post_load
    def post_load(self, data):
        return Label(**data)

class Error(object):
    def __init__(self, **kwargs):
        self.id = kwargs.get('id', None)
        self.name = kwargs.get('name', None)
        self.index = kwargs.get('index', None)
        self.description = kwargs.get('description', None)

class ErrorSchema(Schema):
    id = fields.Int(missing=None)
    name = fields.Str()
    index = fields.Int()
    description = fields.Str()

    @post_load
    def post_load(self, data):
        return Error(**data)

class Testvar(object):
    def __init__(self, **kwargs):
        self.id = kwargs.get('id', None)
        self.name = kwargs.get('name', None)
        self.index = kwargs.get('index', None)
        self.humanname = kwargs.get('humanname', None)
        self.display = kwargs.get('display', None)
        self.dataclass = kwargs.get('dataclass', None)
        self.addedin = kwargs.get('addedin', None)
        self.deprecatedin = kwargs.get('deprecatedin', None)
        self.obsoletedin = kwargs.get('obsoletedin', None)
        self.min = kwargs.get('min', None)
        self.max = kwargs.get('max', None)
        self.length = kwargs.get('length', None)
        self.description = kwargs.get('description', None)
        self.default = kwargs.get('default', None)
        self.defaultdisabled = kwargs.get('defaultdisabled', None)
        self.dyndefault = kwargs.get('dyndefault', None)
        self.keywords = kwargs.get('keywords', None)
        self.alsoaccept = kwargs.get('alsoaccept', None)
        self.wildcard = kwargs.get('wildcard', None)
        self.instances = kwargs.get('instances', None)
        self.parent = kwargs.get('parent', None)
        self.children = kwargs.get('children', None)
        self.tests = kwargs.get('tests', None)

class TestvarSchema(Schema):
    id = fields.Int(missing=None)
    name = fields.Str()
    index = fields.Int()
    humanname = fields.Str()
    display = fields.Str()
    dataclass = fields.Str()
    addedin = fields.Str()
    deprecatedin = fields.Str()
    obsoletedin = fields.Str()
    min = fields.Str()
    max = fields.Str()
    length = fields.Str()
    description = fields.Str()
    default = fields.Str()
    defaultdisabled = fields.Bool()
    dyndefault = fields.Bool()
    keywords = fields.List(fields.Str())
    alsoaccept = fields.List(fields.Str())
    wildcard = fields.Bool()
    instances = fields.Int()
    parent = fields.Str()
    children = fields.List(fields.Str())
    tests = fields.List(fields.Str())

    @post_load
    def post_load(self, data):
        return Testvar(**data)

class TestsuitesService(object):
    """Service for accessing CDRouter Testsuites."""

    RESOURCE = 'testsuites'
    BASE = RESOURCE + '/1/'

    def __init__(self, service):
        self.service = service
        self.base = self.BASE

    def info(self):
        """Get testsuite info."""
        return self.service.get(self.base)

    def search(self, query):
        """Perform full text search of testsuite."""
        return self.service.get(self.base+'search/', params={'q': query})

    def list_groups(self, filter=None, sort=None): # pylint: disable=redefined-builtin
        """Get a list of groups."""
        schema = GroupSchema()
        resp = self.service.list(self.base+'groups/', filter, sort)
        return self.service.decode(schema, resp, many=True)

    def get_group(self, name):
        """Get a group."""
        schema = GroupSchema()
        resp = self.service.get(self.base+'tests/'+name+'/')
        return self.service.decode(schema, resp)

    def list_modules(self, filter=None, sort=None): # pylint: disable=redefined-builtin
        """Get a list of modules."""
        schema = ModuleSchema(exclude=('index', 'labels'))
        resp = self.service.list(self.base+'modules/', filter, sort)
        return self.service.decode(schema, resp, many=True)

    def get_module(self, name):
        """Get a module."""
        schema = ModuleSchema()
        resp = self.service.get(self.base+'modules/'+name+'/')
        return self.service.decode(schema, resp)

    def list_tests(self, filter=None, sort=None): # pylint: disable=redefined-builtin
        """Get a list of tests."""
        schema = TestSchema(exclude=('description', 'labels', 'testvars', 'skip_name', 'skip_reason'))
        resp = self.service.list(self.base+'tests/', filter, sort)
        return self.service.decode(schema, resp, many=True)

    def get_test(self, name):
        """Get a test."""
        schema = TestSchema(exclude=('skip_name', 'skip_reason'))
        resp = self.service.get(self.base+'tests/'+name+'/')
        return self.service.decode(schema, resp)

    def list_labels(self, filter=None, sort=None): # pylint: disable=redefined-builtin
        """Get a list of labels."""
        schema = LabelSchema(exclude=('index', 'description', 'modules', 'tests'))
        resp = self.service.list(self.base+'labels/', filter, sort)
        return self.service.decode(schema, resp, many=True)

    def get_label(self, name):
        """Get a label."""
        schema = LabelSchema()
        resp = self.service.get(self.base+'labels/'+name+'/')
        return self.service.decode(schema, resp)

    def list_errors(self, filter=None, sort=None): # pylint: disable=redefined-builtin
        """Get a list of errors."""
        schema = ErrorSchema(exclude=('index', 'description'))
        resp = self.service.list(self.base+'errors/', filter, sort)
        return self.service.decode(schema, resp, many=True)

    def get_error(self, name):
        """Get a error."""
        schema = ErrorSchema()
        resp = self.service.get(self.base+'errors/'+name+'/')
        return self.service.decode(schema, resp)

    def list_testvars(self, filter=None, sort=None): # pylint: disable=redefined-builtin
        """Get a list of testvars."""
        schema = TestvarSchema(exclude=('index', 'humanname', 'addedin', 'deprecatedin', 'obsoletedin',
                                        'min', 'max', 'length', 'dyndefault', 'keywords', 'alsoaccept',
                                        'wildcard', 'instances', 'parent', 'children', 'tests'))
        resp = self.service.list(self.base+'testvars/', filter, sort)
        return self.service.decode(schema, resp, many=True)

    def get_testvar(self, name):
        """Get a testvar."""
        schema = TestvarSchema()
        resp = self.service.get(self.base+'testvars/'+name+'/')
        return self.service.decode(schema, resp)



#
# Copyright (c) 2017 by QA Cafe.
# All Rights Reserved.
#

"""Module for accessing CDRouter TestResults."""

from marshmallow import Schema, fields, post_load
from .cdr_datetime import DateTime

class Summary(object):
    def __init__(self, **kwargs):
        self.errors = kwargs.get('errors', None)
        self.fails = kwargs.get('fails', None)
        self.passes = kwargs.get('passes', None)
        self.warnings = kwargs.get('warnings', None)

class SummarySchema(Schema):
    errors = fields.Int()
    fails = fields.Int()
    passes = fields.Int()
    warnings = fields.Int()

    @post_load
    def post_load(self, data):
        return Summary(**data)

class Line(object):
    def __init__(self, **kwargs):
        self.raw = kwargs.get('raw', None)

        self.line = kwargs.get('line', None)
        self.header = kwargs.get('header', None)
        self.section = kwargs.get('section', None)
        self.prefix = kwargs.get('prefix', None)
        self.name = kwargs.get('name', None)
        self.timestamp = kwargs.get('timestamp', None)
        self.message = kwargs.get('message', None)

        self.interface = kwargs.get('interface', None)
        self.packet = kwargs.get('packet', None)

        self.src = kwargs.get('src', None)
        self.dst = kwargs.get('dst', None)
        self.proto = kwargs.get('proto', None)
        self.info = kwargs.get('info', None)

        self.summary = kwargs.get('summary', None)

class LineSchema(Schema):
    raw = fields.Str()

    line = fields.Int()
    header = fields.Bool(missing=None)
    section = fields.Bool(missing=None)
    prefix = fields.Str(missing=None)
    name = fields.Str(missing=None)
    timestamp = fields.Str(missing=None)
    message = fields.Str()

    interface = fields.Str(missing=None)
    packet = fields.Str(missing=None)

    src = fields.Str(missing=None)
    dst = fields.Str(missing=None)
    proto = fields.Str(missing=None)
    info = fields.Str(missing=None)

    summary = fields.Nested(SummarySchema)

    @post_load
    def post_load(self, data):
        return Line(**data)

class Log(object):
    def __init__(self, **kwargs):
        self.offset = kwargs.get('offset', None)
        self.limit = kwargs.get('limit', None)
        self.lines = kwargs.get('lines', None)
        self.total = kwargs.get('total', None)

class LogSchema(Schema):
    offset = fields.Int()
    limit = fields.Int()
    lines = fields.Nested(LineSchema, many=True)
    total = fields.Int()

    @post_load
    def post_load(self, data):
        return Log(**data)

class TestResult(object):
    def __init__(self, **kwargs):
        self.id = kwargs.get('id', None)
        self.seq = kwargs.get('seq', None)
        self.loop = kwargs.get('loop', None)
        self.result = kwargs.get('result', None)
        self.retries = kwargs.get('retries', None)
        self.started = kwargs.get('started', None)
        self.duration = kwargs.get('duration', None)
        self.flagged = kwargs.get('flagged', None)
        self.name = kwargs.get('name', None)
        self.description = kwargs.get('description', None)
        self.skip_name = kwargs.get('skip_name', None)
        self.skip_reason = kwargs.get('skip_reason', None)
        self.log = kwargs.get('log', None)
        self.note = kwargs.get('note', None)

class TestResultSchema(Schema):
    id = fields.Str()
    seq = fields.Str()
    loop = fields.Str()
    result = fields.Str()
    retries = fields.Int()
    started = DateTime()
    duration = fields.Int()
    flagged = fields.Bool()
    name = fields.Str()
    description = fields.Str()
    skip_name = fields.Str()
    skip_reason = fields.Str()
    log = fields.Str()
    note = fields.Str()

    @post_load
    def post_load(self, data):
        return TestResult(**data)

class TestResultsService(object):
    """Service for accessing CDRouter TestResults."""

    RESOURCE = 'tests'
    BASE = '/' + RESOURCE + '/'

    def __init__(self, service):
        self.service = service

    def _base(self, id): # pylint: disable=invalid-name,redefined-builtin
        return 'results/'+str(id)+self.BASE

    def list(self, id, filter=None, sort=None, limit=None, page=None): # pylint: disable=invalid-name,redefined-builtin
        """Get a list of test results."""
        schema = TestResultSchema()
        resp = self.service.list(self._base(id), filter, sort, limit, page)
        return self.service.decode(schema, resp, many=True)

    def list_csv(self, id, filter=None, sort=None, limit=None, page=None): # pylint: disable=invalid-name,redefined-builtin
        """Get a list of test results as CSV."""
        return self.service.list(self._base(id), filter, sort, limit, page, format='csv')

    def get(self, id, seq): # pylint: disable=invalid-name,redefined-builtin
        """Get a test result."""
        schema = TestResultSchema()
        resp = self.service.get_id(self._base(id), seq)
        return self.service.decode(schema, resp)

    def edit(self, id, resource): # pylint: disable=invalid-name,redefined-builtin
        """Edit a test result."""
        schema = TestResultSchema(exclude=('id', 'seq', 'loop', 'result', 'retries', 'started', 'duration', 'name', 'description', 'skip_name', 'skip_reason', 'log'))
        json = self.service.encode(schema, resource)

        schema = TestResultSchema()
        resp = self.service.edit(self._base(id), resource.seq, json)
        return self.service.decode(schema, resp)

    def get_log(self, id, seq, offset=None, limit=None, filter=None, packets=None, timestamp_format=None): # pylint: disable=invalid-name,redefined-builtin
        """Get a test result's log."""
        schema = LogSchema()
        resp = self.service.get(self._base(id)+str(seq)+'/log/',
                                params={'offset': offset, 'limit': limit, 'filter': filter,
                                        'packets': packets, 'timestamp_format': timestamp_format, 'format': 'json'})
        return self.service.decode(schema, resp)

    def get_log_plaintext(self, id, seq): # pylint: disable=invalid-name,redefined-builtin
        """Get a test result's log as plaintext."""
        return self.service.get(self._base(id)+str(seq)+'/log/',
                                params={'format': 'text'})



#
# Copyright (c) 2017 by QA Cafe.
# All Rights Reserved.
#

from setuptools import setup, find_packages
from codecs import open
from os import path
import sys

here = path.abspath(path.dirname(__file__))

sys.path.insert(0, here)

import cdrouter

with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='cdrouter',
    version=cdrouter.__version__,
    description='Python client for the CDRouter Web API',
    long_description=long_description,
    url='https://github.com/qacafe/cdrouter.py',
    author='QA Cafe',
    author_email='support@qacafe.com',
    license='MIT',
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        'License :: OSI Approved',
        'License :: OSI Approved :: MIT License',
        'Intended Audience :: Developers',
        "Environment :: Web Environment",
        "Topic :: Software Development",
        "Operating System :: OS Independent",
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    keywords='cdrouter json rest api client',
    packages=['cdrouter'],
    install_requires=['marshmallow', 'requests', 'requests-toolbelt']
)



#
# Copyright (c) 2017 by QA Cafe.
# All Rights Reserved.
#

"""Module for accessing CDRouter Annotations."""

from marshmallow import Schema, fields, post_load

class Annotation(object):
    def __init__(self, **kwargs):
        self.id = kwargs.get('id', None)
        self.seq = kwargs.get('seq', None)
        self.line = kwargs.get('line', None)
        self.comment = kwargs.get('comment', None)

class AnnotationSchema(Schema):
    id = fields.Str()
    seq = fields.Str()
    line = fields.Str()
    comment = fields.Str()

    @post_load
    def post_load(self, data):
        return Annotation(**data)

class AnnotationsService(object):
    """Service for accessing CDRouter Annotations."""

    RESOURCE = 'annotations'
    BASE = RESOURCE + '/'

    def __init__(self, service):
        self.service = service

    def _base(self, id, seq): # pylint: disable=invalid-name,redefined-builtin
        return 'results/'+str(id)+'/tests/'+str(seq)+self.BASE

    def list(self, id, seq): # pylint: disable=invalid-name,redefined-builtin
        """Get a list of annotations."""
        schema = AnnotationSchema(exclude=('id', 'seq'))
        resp = self.service.list(self._base(id, seq))
        return self.service.decode(schema, resp, many=True)

    def get(self, id, seq, line): # pylint: disable=invalid-name,redefined-builtin
        """Get an annotation."""
        schema = AnnotationSchema()
        resp = self.service.get_id(self._base(id, seq), line)
        return self.service.decode(schema, resp)

    def create_or_edit(self, id, seq, resource): # pylint: disable=invalid-name,redefined-builtin
        """Create or edit an annotation."""
        schema = AnnotationSchema(exclude=('id', 'seq'))
        json = self.service.encode(schema, resource)

        schema = AnnotationSchema()
        resp = self.service.edit(self._base(id, seq), resource.line, json)
        return self.service.decode(schema, resp)

    def create(self, id, seq, resource): # pylint: disable=invalid-name,redefined-builtin
        return self.create_or_edit(id, seq, resource)

    def edit(self, id, seq, resource): # pylint: disable=invalid-name,redefined-builtin
        return self.create_or_edit(id, seq, resource)

    def delete(self, id, seq, line): # pylint: disable=invalid-name,redefined-builtin
        """Delete an annotation."""
        return self.service.delete_id(self._base(id, seq), line)



#
# Copyright (c) 2017 by QA Cafe.
# All Rights Reserved.
#

import io
import re
import requests
from requests_toolbelt.downloadutils import stream
from requests_toolbelt import sessions
from requests_toolbelt.utils.user_agent import user_agent
from requests.packages.urllib3.exceptions import InsecureRequestWarning
from marshmallow import Schema, fields, post_load

from . import __version__
from .cdr_datetime import DateTime
from .configs import ConfigsService
from .devices import DevicesService
from .jobs import JobsService
from .packages import PackagesService
from .results import ResultsService
from .testresults import TestResultsService
from .annotations import AnnotationsService
from .captures import CapturesService
from .highlights import HighlightsService
from .imports import ImportsService
from .exports import ExportsService
from .history import HistoryService
from .system import SystemService
from .tags import TagsService
from .testsuites import TestsuitesService
from .users import UsersService

class Links(object):
    def __init__(self, **kwargs):
        self.first = kwargs.get('first', None)
        self.last = kwargs.get('last', None)
        self.current = kwargs.get('current', None)
        self.total = kwargs.get('total', None)
        self.limit = kwargs.get('limit', None)
        self.next = kwargs.get('next', None)
        self.prev = kwargs.get('prev', None)

class LinksSchema(Schema):
    first = fields.Int()
    last = fields.Int()
    current = fields.Int()
    total = fields.Int()
    limit = fields.Int()
    next = fields.Int(missing=None)
    prev = fields.Int(missing=None)

    @post_load
    def post_load(self, data):
        return Links(**data)

class Response(object):
    def __init__(self, **kwargs):
        self.timestamp = kwargs.get('timestamp', None)
        self.error = kwargs.get('error', None)
        self.data = kwargs.get('data', None)
        self.links = kwargs.get('links', None)

class ResponseSchema(Schema):
    timestamp = DateTime()
    error = fields.Str(missing=None)
    data = fields.Dict(missing=None)

    @post_load
    def post_load(self, data):
        return Response(**data)

class ListResponseSchema(ResponseSchema):
    data = fields.List(fields.Dict(), missing=None)
    links = fields.Nested(LinksSchema, missing=None)

    @post_load
    def post_load(self, data):
        return Response(**data)

class Share(object):
    def __init__(self, **kwargs):
        self.user_id = kwargs.get('user_id', None)
        self.read = kwargs.get('read', None)
        self.write = kwargs.get('write', None)
        self.execute = kwargs.get('execute', None)

class ShareSchema(Schema):
    user_id = fields.Str()
    read = fields.Bool()
    write = fields.Bool()
    execute = fields.Bool()

    @post_load
    def post_load(self, data):
        return Share(**data)

class CDRouterError(BaseException):
    def __init__(self, message, response):
        self.message = message
        self.response = response

    def __str__(self):
        return self.message

class Auth(requests.auth.AuthBase): # pylint: disable=too-few-public-methods
    """Class for authorizing CDRouter Web API requests."""

    def __init__(self, token=None):
        self.token = token

    def __call__(self, r):
        if self.token != None:
            r.headers['authorization'] = 'Bearer ' + self.token
        return r

class CDRouter(object):
    """Service for accessing the CDRouter Web API."""
    BASE = '/api/v1/'

    def __init__(self, base, token=None, insecure=False):
        self.base = base.rstrip('/')+self.BASE
        self.token = token
        self.insecure = insecure

        if insecure:
            # disable annoying InsecureRequestWarning
            requests.packages.urllib3.disable_warnings(InsecureRequestWarning)

        self.session = sessions.BaseUrlSession(base_url=base+self.BASE)

        # resource-specific services
        self.configs = ConfigsService(self)
        self.devices = DevicesService(self)
        self.jobs = JobsService(self)
        self.packages = PackagesService(self)
        self.results = ResultsService(self)
        self.tests = TestResultsService(self)
        self.annotations = AnnotationsService(self)
        self.captures = CapturesService(self)
        self.highlights = HighlightsService(self)
        self.imports = ImportsService(self)
        self.exports = ExportsService(self)
        self.history = HistoryService(self)
        self.system = SystemService(self)
        self.tags = TagsService(self)
        self.testsuites = TestsuitesService(self)
        self.users = UsersService(self)

    # base request methods
    def _req(self, path, method='GET', json=None, data=None, params=None, headers=None, files=None, stream=None):
        if params is None:
            params = {}
        if headers is None:
            headers = {}
        if files is None:
            files = {}
            headers.update({'user-agent': user_agent('cdrouter.py', __version__)})
        return self.session.request(method, path, params=params, headers=headers, files=files, stream=stream,
                                    json=json, data=data, verify=(not self.insecure), auth=Auth(token=self.token))

    def get(self, path, params=None, stream=None):
        """Send an authorized GET request."""
        return self._req(path, method='GET', params=params, stream=None)

    def post(self, path, json=None, data=None, params=None, files=None):
        """Send an authorized POST request."""
        return self._req(path, method='POST', json=json, data=data, params=params, files=files)

    def patch(self, path, json, params=None):
        """Send an authorized PATCH request."""
        return self._req(path, method='PATCH', json=json, params=params)

    def delete(self, path, params=None):
        """Send an authorized DELETE request."""
        return self._req(path, method='DELETE', params=params)

    # cdrouter-specific request methods
    def list(self, base, filter=None, sort=None, limit=None, page=None, format=None): # pylint: disable=redefined-builtin
        """Send an authorized GET request for a collection."""
        if sort != None:
            sort = ','.join(sort)
        return self.get(base, params={'filter': filter, 'sort': sort, 'limit': limit,
                                      'page': page, 'format': format})

    def get_id(self, base, id, params=None): # pylint: disable=invalid-name,redefined-builtin
        """Send an authorized GET request to get a resource by ID."""
        return self.get(base+str(id)+'/', params=params)

    def create(self, base, resource):
        """Send an authorized POST request to create a new resource."""
        return self.post(base, json=resource)

    def edit(self, base, id, resource): # pylint: disable=invalid-name,redefined-builtin
        """Send an authorized PATCH request to edit a resource."""
        return self.patch(base+str(id)+'/', json=resource)

    def delete_id(self, base, id): # pylint: disable=invalid-name,redefined-builtin
        """Send an authorized DELETE request to delete a resource by ID."""
        return self.delete(base+str(id)+'/')

    def get_shares(self, base, id): # pylint: disable=invalid-name,redefined-builtin
        """Send an authorized GET request to get a resource's shares."""
        schema = ShareSchema()
        resp = self.get(base+str(id)+'/shares/')
        return self.decode(schema, resp, many=True)

    def edit_shares(self, base, id, user_ids): # pylint: disable=invalid-name,redefined-builtin
        """Send an authorized PATCH request to edit a resource's shares."""
        schema = ShareSchema()
        resp = self.patch(base+str(id)+'/shares/', json={'user_ids': map(int, user_ids)})
        return self.decode(schema, resp, many=True)

    def filename(self, resp, filename=None):
        if 'content-disposition' in resp.headers:
            m = re.search('filename="([^"]+)"', resp.headers['content-disposition'])
            if m is not None:
                filename = m.group(1)
        return filename

    def export(self, base, id, format='gz', params=None): # pylint: disable=invalid-name,redefined-builtin
        """Send an authorized GET request to export a resource."""
        if params is None:
            params = {}
        params.update({'format': format})
        resp = self.get(base+str(id)+'/', params=params, stream=True)
        resp.raise_for_status()
        b = io.BytesIO()
        stream.stream_response_to_file(resp, path=b)
        return (b, self.filename(resp))

    def bulk_export(self, base, ids, params=None):
        """Send an authorized GET request to bulk export a set of resources."""
        if params is None:
            params = {}
        params.update({'bulk': 'export', 'ids': map(str, ids)})
        resp = self.get(base, params=params, stream=True)
        resp.raise_for_status()
        b = io.BytesIO()
        stream.stream_response_to_file(resp, path=b)
        return (b, self.filename(resp))

    def bulk_copy(self, base, resource, ids, schema):
        """Send an authorized POST request to bulk copy a set of resources."""
        resp = self.post(base, params={'bulk': 'copy'},
                         json={resource: [{'id': str(x)} for x in ids]})
        return self.decode(schema, resp, many=True)

    def bulk_edit(self, base, resource, fields, ids=None, filter=None, all=False, testvars=None): # pylint: disable=redefined-builtin
        """Send an authorized POST request to bulk edit a set of resources."""
        json = {'fields': fields}
        if ids != None or testvars != None:
            if ids != None:
                json[resource] = [{'id': str(x)} for x in ids]
            if testvars != None:
                json['testvars'] = testvars

        return self.post(base, params={'bulk': 'edit', 'filter': filter, 'all': all}, json=json)

    def bulk_delete(self, base, resource, ids=None, filter=None, all=False): # pylint: disable=redefined-builtin
        """Send an authorized POST request to bulk delete a set of resources."""
        json = None
        if ids != None:
            json = {resource: [{'id': str(x)} for x in ids]}
        return self.post(base, params={'bulk': 'delete', 'filter': filter, 'all': all}, json=json)

    def decode(self, schema, resp, many=None, links=False):
        json = resp.json()
        resp_schema = ResponseSchema()
        if many is True:
            resp_schema = ListResponseSchema()

        result = resp_schema.load(json).data

        if 400 <= resp.status_code < 600 and result.error is not None:
                raise CDRouterError(result.error, resp)

        resp.raise_for_status()

        if result.data is None:
            raise CDRouterError('no data field in JSON response!', resp)

        data = schema.load(result.data, many=many).data

        if many is True and links is True and result.links is not None:
            return (data, result.links)

        return data

    def encode(self, schema, resource, many=None):
        result = schema.dump(resource, many=many)
        return result.data



#
# Copyright (c) 2017 by QA Cafe.
# All Rights Reserved.
#

"""Module for accessing CDRouter System."""

import os.path

from marshmallow import Schema, fields, post_load

class Version(object):
    def __init__(self, **kwargs):
        self.raw = kwargs.get('raw', None)
        self.major = kwargs.get('major', None)
        self.minor = kwargs.get('minor', None)
        self.build = kwargs.get('build', None)

class VersionSchema(Schema):
    raw = fields.Str()
    major = fields.Int()
    minor = fields.Int()
    build = fields.Int()

    @post_load
    def post_load(self, data):
        return Version(**data)

class ReleaseLatest(object):
    def __init__(self, **kwargs):
        self.latest = kwargs.get('latest', None)
        self.current = kwargs.get('current', None)
        self.newer = kwargs.get('newer', None)

class ReleaseLatestSchema(Schema):
    latest = fields.Nested(VersionSchema)
    current = fields.Nested(VersionSchema)
    newer = fields.Bool()

    @post_load
    def post_load(self, data):
        return ReleaseLatest(**data)

class Testsuite(object):
    def __init__(self, **kwargs):
        self.shortname = kwargs.get('shortname', None)
        self.name = kwargs.get('name', None)

class TestsuiteSchema(Schema):
    shortname = fields.Str()
    name = fields.Str()

    @post_load
    def post_load(self, data):
        return Testsuite(**data)

class Release(object):
    def __init__(self, **kwargs):
        self.build_date = kwargs.get('build_date', None)
        self.filename = kwargs.get('filename', None)
        self.version = kwargs.get('version', None)
        self.id = kwargs.get('id', None)
        self.article_id = kwargs.get('article_id', None)
        self.testsuite = kwargs.get('testsuite', None)

class ReleaseSchema(Schema):
    build_date = fields.Str()
    filename = fields.Str()
    version = fields.Nested(VersionSchema)
    id = fields.Int()
    article_id = fields.Int()
    testsuite = fields.Nested(TestsuiteSchema)

    @post_load
    def post_load(self, data):
        return Release(**data)

class Upgrade(object):
    def __init__(self, **kwargs):
        self.success = kwargs.get('success', None)
        self.installer_path = kwargs.get('installer_path', None)
        self.output = kwargs.get('output', None)
        self.error = kwargs.get('error', None)

class UpgradeSchema(Schema):
    success = fields.Bool()
    installer_path = fields.Str(missing=None)
    output = fields.Str(missing=None)
    error = fields.Str(missing=None)

    @post_load
    def post_load(self, data):
        return Upgrade(**data)

class InterfaceFlags(object):
    def __init__(self, **kwargs):
        self.up = kwargs.get('up', None)
        self.broadcast = kwargs.get('broadcast', None)
        self.loopback = kwargs.get('loopback', None)
        self.point_to_point = kwargs.get('point_to_point', None)
        self.multicast = kwargs.get('multicast', None)

class InterfaceFlagsSchema(Schema):
    up = fields.Bool()
    broadcast = fields.Bool()
    loopback = fields.Bool()
    point_to_point = fields.Bool()
    multicast = fields.Bool()

    @post_load
    def post_load(self, data):
        return InterfaceFlags(**data)

class InterfaceAddr(object):
    def __init__(self, **kwargs):
        self.network = kwargs.get('network', None)
        self.address = kwargs.get('address', None)

class InterfaceAddrSchema(Schema):
    network = fields.Str()
    address = fields.Str()

    @post_load
    def post_load(self, data):
        return InterfaceAddr(**data)

class Interface(object):
    def __init__(self, **kwargs):
        self.index = kwargs.get('index', None)
        self.mtu = kwargs.get('mtu', None)
        self.name = kwargs.get('name', None)
        self.hardware_addr = kwargs.get('hardware_addr', None)
        self.flags = kwargs.get('flags', None)
        self.addresses = kwargs.get('addresses', None)
        self.multicast_addresses = kwargs.get('multicast_addresses', None)

class InterfaceSchema(Schema):
    index = fields.Str()
    mtu = fields.Str()
    name = fields.Str()
    hardware_addr = fields.Str()
    flags = fields.Nested(InterfaceFlagsSchema)
    addresses = fields.Nested(InterfaceAddrSchema, many=True, missing=None)
    multicast_addresses = fields.Nested(InterfaceAddrSchema, many=True, missing=None)

    @post_load
    def post_load(self, data):
        return Interface(**data)

class Preferences(object):
    def __init__(self, **kwargs):
        self.automatic_login = kwargs.get('automatic_login', None)
        self.cloudshark_appliance_autotags = kwargs.get('cloudshark_appliance_autotags', None)
        self.cloudshark_appliance_insecure = kwargs.get('cloudshark_appliance_insecure', None)
        self.cloudshark_appliance_password = kwargs.get('cloudshark_appliance_password', None)
        self.cloudshark_appliance_tags = kwargs.get('cloudshark_appliance_tags', None)
        self.cloudshark_appliance_token = kwargs.get('cloudshark_appliance_token', None)
        self.cloudshark_appliance_url = kwargs.get('cloudshark_appliance_url', None)
        self.cloudshark_appliance_username = kwargs.get('cloudshark_appliance_username', None)
        self.hostname = kwargs.get('hostname', None)
        self.migrated = kwargs.get('migrated', None)
        self.port = kwargs.get('port', None)
        self.https = kwargs.get('https', None)
        self.force_https = kwargs.get('force_https', None)
        self.use_cloudshark = kwargs.get('use_cloudshark', None)
        self.log_timestamp_format = kwargs.get('log_timestamp_format', None)
        self.editor_keymap = kwargs.get('editor_keymap', None)
        self.lounge_url = kwargs.get('lounge_url', None)
        self.lounge_insecure = kwargs.get('lounge_insecure', None)

class PreferencesSchema(Schema):
    automatic_login = fields.Bool()
    cloudshark_appliance_autotags = fields.Str()
    cloudshark_appliance_insecure = fields.Str()
    cloudshark_appliance_password = fields.Str()
    cloudshark_appliance_tags = fields.Str()
    cloudshark_appliance_token = fields.Str()
    cloudshark_appliance_url = fields.Str()
    cloudshark_appliance_username = fields.Str()
    hostname = fields.Str()
    migrated = fields.List(fields.Str())
    port = fields.Int()
    https = fields.Int()
    force_https = fields.Str()
    use_cloudshark = fields.Str()
    log_timestamp_format = fields.Str()
    editor_keymap = fields.Str()
    lounge_url = fields.Str()
    lounge_insecure = fields.Bool()

    @post_load
    def post_load(self, data):
        return Preferences(**data)

class SystemService(object):
    """Service for accessing CDRouter System."""

    RESOURCE = 'system'
    BASE = RESOURCE + '/'

    def __init__(self, service):
        self.service = service
        self.base = self.BASE

    def latest_lounge_release(self):
        """Get the latest release of CDRouter from the CDRouter Support Lounge."""
        schema = ReleaseLatestSchema()
        resp = self.service.get(self.base+'lounge/latest/')
        return self.service.decode(schema, resp)

    def check_for_lounge_upgrade(self, email, password):
        """Check the CDRouter Support Lounge for eligible upgrades using your
        Support Lounge email & password."""
        schema = ReleaseSchema()
        resp = self.service.post(self.base+'lounge/check/',
                                 json={'email': email, 'password': password})
        return self.service.decode(schema, resp)

    def lounge_upgrade(self, email, password, release_id):
        """Download & install an upgrade from the CDRouter Support Lounge
        using your Support Lounge email & password. Please note that any
        running tests will be stopped."""
        schema = UpgradeSchema()
        resp = self.service.post(self.base+'lounge/upgrade/',
                                 json={'email': email, 'password': password, 'release': {'id': int(release_id)}})
        return self.service.decode(schema, resp)

    def manual_upgrade(self, filepath):
        """Upgrade CDRouter manually by uploading a .bin installer from the
        CDRouter Support Lounge. Please note that any running tests will be
        stopped."""
        schema = UpgradeSchema()
        resp = self.service.post(self.base+'upgrade/',
                                 files={'file': (os.path.basename(filepath), open(filepath, 'rb'))})
        return self.service.decode(schema, resp)

    def restart(self):
        """Restart CDRouter web interface. Please note that any running tests will be stopped."""
        return self.service.post(self.base+'restart/')

    def time(self):
        """Get system time."""
        return self.service.post(self.base+'time/').json()['data']['time']

    def hostname(self):
        """Get system hostname."""
        return self.service.post(self.base+'hostname/').json()['data']

    def interfaces(self, addresses=False):
        """Get system interfaces."""
        schema = InterfaceSchema()
        resp = self.service.get(self.base+'interfaces/', params={'addresses': addresses})
        return self.service.decode(schema, resp, many=True)

    def get_preferences(self):
        """Get preferences from /usr/cdrouter-data/etc/config.yml."""
        schema = PreferencesSchema()
        resp = self.service.get(self.base+'preferences/')
        return self.service.decode(schema, resp)

    def edit_preferences(self, resource):
        """Edit preferences in /usr/cdrouter-data/etc/config.yml."""
        schema = PreferencesSchema()
        json = self.service.encode(schema, resource)

        schema = PreferencesSchema()
        resp = self.service.patch(self.base+'preferences/', json=json)
        return self.service.decode(schema, resp)



#
# Copyright (c) 2017 by QA Cafe.
# All Rights Reserved.
#

import os.path

from marshmallow import Schema, fields, post_load
from .cdr_datetime import DateTime

class Attachment(object):
    def __init__(self, **kwargs):
        self.id = kwargs.get('id', None)
        self.name = kwargs.get('name', None)
        self.description = kwargs.get('description', None)
        self.created = kwargs.get('created', None)
        self.updated = kwargs.get('updated', None)
        self.size = kwargs.get('size', None)
        self.path = kwargs.get('path', None)
        self.device_id = kwargs.get('device_id', None)

class AttachmentSchema(Schema):
    id = fields.Str()
    name = fields.Str()
    description = fields.Str()
    created = DateTime()
    updated = DateTime()
    size = fields.Int()
    path = fields.Str()
    device_id = fields.Str()

    @post_load
    def post_load(self, data):
        return Attachment(**data)

class AttachmentsService(object):
    RESOURCE = 'attachments'
    BASE = RESOURCE + '/'

    def __init__(self, service):
        self.service = service

    def _base(self, id): # pylint: disable=invalid-name,redefined-builtin
        return 'devices/'+str(id)+self.BASE

    def list(self, id, filter=None, sort=None, limit=None, page=None): # pylint: disable=invalid-name,redefined-builtin
        schema = AttachmentSchema(exclude=('path'))
        resp = self.service.list(self._base(id), filter, sort, limit, page)
        return self.service.decode(schema, resp, many=True)

    def get(self, id, attid): # pylint: disable=invalid-name,redefined-builtin
        schema = AttachmentSchema()
        resp = self.service.get_id(self._base(id), attid)
        return self.service.decode(schema, resp)

    def create(self, id, filepath): # pylint: disable=invalid-name,redefined-builtin
        schema = AttachmentSchema()
        resp = self.service.post(self._base(id),
                                 files={'file': (os.path.basename(filepath), open(filepath, 'rb'))})
        return self.service.decode(schema, resp)

    def download(self, id, attid): # pylint: disable=invalid-name,redefined-builtin
        return self.service.get_id(self._base(id), attid, params={'format': 'download'})

    def thumbnail(self, id, attid, size=None): # pylint: disable=invalid-name,redefined-builtin
        return self.service.get_id(self._base(id), attid, params={'format': 'thumbnail', 'size': size})

    def edit(self, id, resource): # pylint: disable=invalid-name,redefined-builtin
        schema = AttachmentSchema(exclude=('id', 'created', 'updated', 'size', 'path', 'device_id'))
        json = self.service.encode(schema, resource)

        schema = AttachmentSchema()
        resp = self.service.edit(self._base(id), resource.id, json)
        return self.service.decode(schema, resp)

    def delete(self, id, attid): # pylint: disable=invalid-name,redefined-builtin
        return self.service.edit(self._base(id), attid)



#
# Copyright (c) 2017 by QA Cafe.
# All Rights Reserved.
#

"""Module for processing CDRouter datetime strings."""

from datetime import datetime

from marshmallow import fields
from marshmallow.exceptions import ValidationError

class DateTime(fields.DateTime):
    def __init__(self, format=None, **kwargs):
        super(DateTime, self).__init__(format=format, **kwargs)
        
    def _serialize(self, value, attr, obj):
        try:
            return super(DateTime, self)._serialize(value, attr, obj)
        except ValidationError:
            return datetime.min

    def _deserialize(self, value, attr, data):
        try:
            return super(DateTime, self)._deserialize(value, attr, data)
        except ValidationError:
            return datetime.min
